@@ -55,6 +55,10 @@ link_files = {
                 pattern=r'(Python #|bpo-)(?P<python>\d+)',
                 url='https://bugs.python.org/issue{python}',
             ),
+            dict(
+                pattern=r'\bpython/cpython#(?P<cpython>\d+)',
+                url='{GH}/python/cpython/issues/{cpython}',
+            ),
             dict(
                 pattern=r'Interop #(?P<interop>\d+)',
                 url='{GH}/pypa/interoperability-peps/issues/{interop}',
@@ -100,19 +104,17 @@ intersphinx_mapping = {
 # Preserve authored syntax for defaults
 autodoc_preserve_defaults = True
 
-intersphinx_mapping.update(
-    {
-        'pip': ('https://pip.pypa.io/en/latest', None),
-        'build': ('https://pypa-build.readthedocs.io/en/latest', None),
-        'PyPUG': ('https://packaging.python.org/en/latest/', None),
-        'packaging': ('https://packaging.pypa.io/en/latest/', None),
-        'twine': ('https://twine.readthedocs.io/en/stable/', None),
-        'importlib-resources': (
-            'https://importlib-resources.readthedocs.io/en/latest',
-            None,
-        ),
-    }
-)
+intersphinx_mapping.update({
+    'pip': ('https://pip.pypa.io/en/latest', None),
+    'build': ('https://pypa-build.readthedocs.io/en/latest', None),
+    'PyPUG': ('https://packaging.python.org/en/latest/', None),
+    'packaging': ('https://packaging.pypa.io/en/latest/', None),
+    'twine': ('https://twine.readthedocs.io/en/stable/', None),
+    'importlib-resources': (
+        'https://importlib-resources.readthedocs.io/en/latest',
+        None,
+    ),
+})
 
 # Add support for linking usernames
 github_url = 'https://github.com'
@@ -163,22 +165,23 @@ extensions += ['sphinx_inline_tabs']
 # Ref: https://stackoverflow.com/a/30624034/595220
 nitpick_ignore = [
     ('c:func', 'SHGetSpecialFolderPath'),  # ref to MS docs
+    ('envvar', 'DIST_EXTRA_CONFIG'),  # undocumented
     ('envvar', 'DISTUTILS_DEBUG'),  # undocumented
     ('envvar', 'HOME'),  # undocumented
     ('envvar', 'PLAT'),  # undocumented
-    ('envvar', 'DIST_EXTRA_CONFIG'),  # undocumented
     ('py:attr', 'CCompiler.language_map'),  # undocumented
     ('py:attr', 'CCompiler.language_order'),  # undocumented
-    ('py:class', 'distutils.dist.Distribution'),  # undocumented
-    ('py:class', 'distutils.extension.Extension'),  # undocumented
     ('py:class', 'BorlandCCompiler'),  # undocumented
     ('py:class', 'CCompiler'),  # undocumented
     ('py:class', 'CygwinCCompiler'),  # undocumented
+    ('py:class', 'distutils.dist.Distribution'),  # undocumented
     ('py:class', 'distutils.dist.DistributionMetadata'),  # undocumented
+    ('py:class', 'distutils.extension.Extension'),  # undocumented
     ('py:class', 'FileList'),  # undocumented
     ('py:class', 'IShellLink'),  # ref to MS docs
     ('py:class', 'MSVCCompiler'),  # undocumented
     ('py:class', 'OptionDummy'),  # undocumented
+    ('py:class', 'setuptools.dist.Distribution'),  # undocumented
     ('py:class', 'UnixCCompiler'),  # undocumented
     ('py:exc', 'CompileError'),  # undocumented
     ('py:exc', 'DistutilsExecError'),  # undocumented
@@ -188,8 +191,7 @@ nitpick_ignore = [
     ('py:exc', 'PreprocessError'),  # undocumented
     ('py:exc', 'setuptools.errors.PlatformError'),  # sphinx cannot find it
     ('py:func', 'distutils.CCompiler.new_compiler'),  # undocumented
-    # undocumented:
-    ('py:func', 'distutils.dist.DistributionMetadata.read_pkg_file'),
+    ('py:func', 'distutils.dist.DistributionMetadata.read_pkg_file'),  # undocumented
     ('py:func', 'distutils.file_util._copy_file_contents'),  # undocumented
     ('py:func', 'distutils.log.debug'),  # undocumented
     ('py:func', 'distutils.spawn.find_executable'),  # undocumented
