@@ -13,7 +13,7 @@ import distutils.core
 
 import pytest
 import jaraco.path
-from path import Path as _Path
+from path import Path
 
 from .contexts import quiet
 from .integration.helpers import get_sdist_members, get_wheel_members, run
@@ -43,35 +43,27 @@ class TestDiscoverPackagesAndPyModules:
     """Make sure discovered values for ``packages`` and ``py_modules`` work
     similarly to explicit configuration for the simple scenarios.
     """
+
     OPTIONS = {
         # Different options according to the circumstance being tested
-        "explicit-src": {
-            "package_dir": {"": "src"},
-            "packages": ["pkg"]
-        },
+        "explicit-src": {"package_dir": {"": "src"}, "packages": ["pkg"]},
         "variation-lib": {
             "package_dir": {"": "lib"},  # variation of the source-layout
         },
-        "explicit-flat": {
-            "packages": ["pkg"]
-        },
-        "explicit-single_module": {
-            "py_modules": ["pkg"]
-        },
-        "explicit-namespace": {
-            "packages": ["ns", "ns.pkg"]
-        },
+        "explicit-flat": {"packages": ["pkg"]},
+        "explicit-single_module": {"py_modules": ["pkg"]},
+        "explicit-namespace": {"packages": ["ns", "ns.pkg"]},
         "automatic-src": {},
         "automatic-flat": {},
         "automatic-single_module": {},
-        "automatic-namespace": {}
+        "automatic-namespace": {},
     }
     FILES = {
         "src": ["src/pkg/__init__.py", "src/pkg/main.py"],
         "lib": ["lib/pkg/__init__.py", "lib/pkg/main.py"],
         "flat": ["pkg/__init__.py", "pkg/main.py"],
         "single_module": ["pkg.py"],
-        "namespace": ["ns/pkg/__init__.py"]
+        "namespace": ["ns/pkg/__init__.py"],
     }
 
     def _get_info(self, circumstance):
@@ -164,7 +156,7 @@ class TestDiscoverPackagesAndPyModules:
             requires = []
             build-backend = 'setuptools.build_meta'
             """
-        )
+        ),
     }
 
     @pytest.mark.parametrize(
@@ -172,8 +164,8 @@ class TestDiscoverPackagesAndPyModules:
         product(
             ["setup.cfg", "setup.py", "pyproject.toml"],
             ["packages", "py_modules"],
-            FILES.keys()
-        )
+            FILES.keys(),
+        ),
     )
     def test_purposefully_empty(self, tmp_path, config_file, param, circumstance):
         files = self.FILES[circumstance] + ["mod.py", "other.py", "src/pkg/__init__.py"]
@@ -184,11 +176,11 @@ class TestDiscoverPackagesAndPyModules:
         else:
             # Make sure build works with or without setup.cfg
             pyproject = self.PURPOSEFULLY_EMPY["template-pyproject.toml"]
-            (tmp_path / "pyproject.toml").write_text(pyproject)
+            (tmp_path / "pyproject.toml").write_text(pyproject, encoding="utf-8")
             template_param = param
 
         config = self.PURPOSEFULLY_EMPY[config_file].format(param=template_param)
-        (tmp_path / config_file).write_text(config)
+        (tmp_path / config_file).write_text(config, encoding="utf-8")
 
         dist = _get_dist(tmp_path, {})
         # When either parameter package or py_modules is an empty list,
@@ -211,11 +203,11 @@ class TestDiscoverPackagesAndPyModules:
             (
                 # Just the top-level package can have `-stubs`, ignore nested ones
                 ["namespace-stubs/pkg-stubs/__init__.pyi"],
-                {"pkg", "namespace-stubs"}
+                {"pkg", "namespace-stubs"},
             ),
             (["_hidden/file.py"], {"pkg"}),
             (["news/finalize.py"], {"pkg"}),
-        ]
+        ],
     )
     def test_flat_layout_with_extra_files(self, tmp_path, extra_files, pkgs):
         files = self.FILES["flat"] + extra_files
@@ -228,7 +220,7 @@ class TestDiscoverPackagesAndPyModules:
         [
             ["other/__init__.py"],
             ["other/finalize.py"],
-        ]
+        ],
     )
     def test_flat_layout_with_dangerous_extra_files(self, tmp_path, extra_files):
         files = self.FILES["flat"] + extra_files
@@ -295,16 +287,18 @@ class TestWithAttrDirective:
         [
             ("src", {}),
             ("lib", {"packages": "find:", "packages.find": {"where": "lib"}}),
-        ]
+        ],
     )
     def test_setupcfg_metadata(self, tmp_path, folder, opts):
         files = [f"{folder}/pkg/__init__.py", "setup.cfg"]
         _populate_project_dir(tmp_path, files, opts)
-        (tmp_path / folder / "pkg/__init__.py").write_text("version = 42")
-        (tmp_path / "setup.cfg").write_text(
-            "[metadata]\nversion = attr: pkg.version\n"
-            + (tmp_path / "setup.cfg").read_text()
-        )
+
+        config = (tmp_path / "setup.cfg").read_text(encoding="utf-8")
+        overwrite = {
+            folder: {"pkg": {"__init__.py": "version = 42"}},
+            "setup.cfg": "[metadata]\nversion = attr: pkg.version\n" + config,
+        }
+        jaraco.path.build(overwrite, prefix=tmp_path)
 
         dist = _get_dist(tmp_path, {})
         assert dist.get_name() == "pkg"
@@ -312,7 +306,7 @@ class TestWithAttrDirective:
         assert dist.package_dir
         package_path = find_package_path("pkg", dist.package_dir, tmp_path)
         assert os.path.exists(package_path)
-        assert folder in _Path(package_path).parts()
+        assert folder in Path(package_path).parts()
 
         _run_build(tmp_path, "--sdist")
         dist_file = tmp_path / "dist/pkg-42.tar.gz"
@@ -320,11 +314,16 @@ class TestWithAttrDirective:
 
     def test_pyproject_metadata(self, tmp_path):
         _populate_project_dir(tmp_path, ["src/pkg/__init__.py"], {})
-        (tmp_path / "src/pkg/__init__.py").write_text("version = 42")
-        (tmp_path / "pyproject.toml").write_text(
-            "[project]\nname = 'pkg'\ndynamic = ['version']\n"
-            "[tool.setuptools.dynamic]\nversion = {attr = 'pkg.version'}\n"
-        )
+
+        overwrite = {
+            "src": {"pkg": {"__init__.py": "version = 42"}},
+            "pyproject.toml": (
+                "[project]\nname = 'pkg'\ndynamic = ['version']\n"
+                "[tool.setuptools.dynamic]\nversion = {attr = 'pkg.version'}\n"
+            ),
+        }
+        jaraco.path.build(overwrite, prefix=tmp_path)
+
         dist = _get_dist(tmp_path, {})
         assert dist.get_version() == "42"
         assert dist.package_dir == {"": "src"}
@@ -362,7 +361,7 @@ class TestWithCExtension:
             ]
             setup(ext_modules=ext_modules)
         """
-        (tmp_path / "setup.py").write_text(DALS(setup_script))
+        (tmp_path / "setup.py").write_text(DALS(setup_script), encoding="utf-8")
 
     def test_skip_discovery_with_setupcfg_metadata(self, tmp_path):
         """Ensure that auto-discovery is not triggered when the project is based on
@@ -375,14 +374,14 @@ class TestWithCExtension:
             requires = []
             build-backend = 'setuptools.build_meta'
         """
-        (tmp_path / "pyproject.toml").write_text(DALS(pyproject))
+        (tmp_path / "pyproject.toml").write_text(DALS(pyproject), encoding="utf-8")
 
         setupcfg = """
             [metadata]
             name = proj
             version = 42
         """
-        (tmp_path / "setup.cfg").write_text(DALS(setupcfg))
+        (tmp_path / "setup.cfg").write_text(DALS(setupcfg), encoding="utf-8")
 
         dist = _get_dist(tmp_path, {})
         assert dist.get_name() == "proj"
@@ -407,7 +406,7 @@ class TestWithCExtension:
             name = 'proj'
             version = '42'
         """
-        (tmp_path / "pyproject.toml").write_text(DALS(pyproject))
+        (tmp_path / "pyproject.toml").write_text(DALS(pyproject), encoding="utf-8")
         with pytest.raises(PackageDiscoveryError, match="multiple (packages|modules)"):
             _get_dist(tmp_path, {})
 
@@ -424,7 +423,7 @@ class TestWithPackageData:
         manifest = """
             global-include *.py *.txt
         """
-        (tmp_path / "MANIFEST.in").write_text(DALS(manifest))
+        (tmp_path / "MANIFEST.in").write_text(DALS(manifest), encoding="utf-8")
 
     EXAMPLE_SETUPCFG = """
     [metadata]
@@ -455,7 +454,8 @@ class TestWithPackageData:
             (
                 "src",
                 {
-                    "setup.cfg": DALS(EXAMPLE_SETUPCFG) + DALS(
+                    "setup.cfg": DALS(EXAMPLE_SETUPCFG)
+                    + DALS(
                         """
                         packages = find:
                         package_dir =
@@ -465,12 +465,13 @@ class TestWithPackageData:
                         where = src
                         """
                     )
-                }
+                },
             ),
             (
                 "src",
                 {
-                    "pyproject.toml": DALS(EXAMPLE_PYPROJECT) + DALS(
+                    "pyproject.toml": DALS(EXAMPLE_PYPROJECT)
+                    + DALS(
                         """
                         [tool.setuptools]
                         package-dir = {"" = "src"}
@@ -478,7 +479,7 @@ class TestWithPackageData:
                     )
                 },
             ),
-        ]
+        ],
     )
     def test_include_package_data(self, tmp_path, src_root, files):
         """
@@ -540,13 +541,15 @@ def test_preserve_explicit_name_with_dynamic_version(tmpdir_cwd, monkeypatch):
         "src": {
             "pkg": {"__init__.py": "__version__ = 42\n"},
         },
-        "pyproject.toml": DALS("""
+        "pyproject.toml": DALS(
+            """
             [project]
             name = "myproj"  # purposefully different from package name
             dynamic = ["version"]
             [tool.setuptools.dynamic]
             version = {"attr" = "pkg.__version__"}
-            """)
+            """
+        ),
     }
     jaraco.path.build(files)
     dist = Distribution({})
@@ -568,9 +571,12 @@ def _populate_project_dir(root, files, options):
     # NOTE: Currently pypa/build will refuse to build the project if no
     # `pyproject.toml` or `setup.py` is found. So it is impossible to do
     # completely "config-less" projects.
-    (root / "setup.py").write_text("import setuptools\nsetuptools.setup()")
-    (root / "README.md").write_text("# Example Package")
-    (root / "LICENSE").write_text("Copyright (c) 2018")
+    basic = {
+        "setup.py": "import setuptools\nsetuptools.setup()",
+        "README.md": "# Example Package",
+        "LICENSE": "Copyright (c) 2018",
+    }
+    jaraco.path.build(basic, prefix=root)
     _write_setupcfg(root, options)
     paths = (root / f for f in files)
     for path in paths:
@@ -595,10 +601,10 @@ def _write_setupcfg(root, options):
             setupcfg["options"][key] = "\n" + str_value
         else:
             setupcfg["options"][key] = str(value)
-    with open(root / "setup.cfg", "w") as f:
+    with open(root / "setup.cfg", "w", encoding="utf-8") as f:
         setupcfg.write(f)
     print("~~~~~ setup.cfg ~~~~~")
-    print((root / "setup.cfg").read_text())
+    print((root / "setup.cfg").read_text(encoding="utf-8"))
 
 
 def _run_build(path, *flags):
@@ -611,14 +617,14 @@ def _get_dist(dist_path, attrs):
 
     script = dist_path / 'setup.py'
     if script.exists():
-        with _Path(dist_path):
+        with Path(dist_path):
             dist = distutils.core.run_setup("setup.py", {}, stop_after="init")
     else:
         dist = Distribution(attrs)
 
     dist.src_root = root
     dist.script_name = "setup.py"
-    with _Path(dist_path):
+    with Path(dist_path):
         dist.parse_config_files()
 
     dist.set_defaults()
@@ -631,7 +637,7 @@ def _run_sdist_programatically(dist_path, attrs):
     cmd.ensure_finalized()
     assert cmd.distribution.packages or cmd.distribution.py_modules
 
-    with quiet(), _Path(dist_path):
+    with quiet(), Path(dist_path):
         cmd.run()
 
     return dist, cmd
