@@ -11,9 +11,10 @@ from textwrap import dedent
 from unittest.mock import Mock
 from uuid import uuid4
 
+from distutils.core import run_setup
+
 import jaraco.envs
 import jaraco.path
-import pip_run.launch
 import pytest
 from path import Path as _Path
 
@@ -23,6 +24,7 @@ from setuptools._importlib import resources as importlib_resources
 from setuptools.command.editable_wheel import (
     _DebuggingTips,
     _LinkTree,
+    _encode_pth,
     _find_virtual_namespaces,
     _find_namespaces,
     _find_package_roots,
@@ -31,6 +33,7 @@ from setuptools.command.editable_wheel import (
 )
 from setuptools.dist import Distribution
 from setuptools.extension import Extension
+from setuptools.warnings import SetuptoolsDeprecationWarning
 
 
 @pytest.fixture(params=["strict", "lenient"])
@@ -41,7 +44,8 @@ def editable_opts(request):
 
 
 EXAMPLE = {
-    'pyproject.toml': dedent("""\
+    'pyproject.toml': dedent(
+        """\
         [build-system]
         requires = ["setuptools"]
         build-backend = "setuptools.build_meta"
@@ -56,8 +60,7 @@ EXAMPLE = {
             "Development Status :: 5 - Production/Stable",
             "Intended Audience :: Developers"
         ]
-        urls = {Homepage = "http://github.com"}
-        dependencies = ['importlib-metadata; python_version<"3.8"']
+        urls = {Homepage = "https://github.com"}
 
         [tool.setuptools]
         package-dir = {"" = "src"}
@@ -69,42 +72,45 @@ EXAMPLE = {
 
         [tool.distutils.egg_info]
         tag-build = ".post0"
-        """),
-    "MANIFEST.in": dedent("""\
+        """
+    ),
+    "MANIFEST.in": dedent(
+        """\
         global-include *.py *.txt
         global-exclude *.py[cod]
         prune dist
         prune build
-        """).strip(),
+        """
+    ).strip(),
     "README.rst": "This is a ``README``",
     "LICENSE.txt": "---- placeholder MIT license ----",
     "src": {
         "mypkg": {
-            "__init__.py": dedent("""\
+            "__init__.py": dedent(
+                """\
                 import sys
-
-                if sys.version_info[:2] >= (3, 8):
-                    from importlib.metadata import PackageNotFoundError, version
-                else:
-                    from importlib_metadata import PackageNotFoundError, version
+                from importlib.metadata import PackageNotFoundError, version
 
                 try:
                     __version__ = version(__name__)
                 except PackageNotFoundError:
                     __version__ = "unknown"
-                """),
-            "__main__.py": dedent("""\
+                """
+            ),
+            "__main__.py": dedent(
+                """\
                 from importlib.resources import read_text
                 from . import __version__, __name__ as parent
                 from .mod import x
 
                 data = read_text(parent, "data.txt")
                 print(__version__, data, x)
-                """),
+                """
+            ),
             "mod.py": "x = ''",
             "data.txt": "Hello World",
         }
-    }
+    },
 }
 
 
@@ -114,32 +120,40 @@ SETUP_SCRIPT_STUB = "__import__('setuptools').setup()"
 @pytest.mark.parametrize(
     "files",
     [
-        {**EXAMPLE, "setup.py": SETUP_SCRIPT_STUB},  # type: ignore
+        {**EXAMPLE, "setup.py": SETUP_SCRIPT_STUB},
         EXAMPLE,  # No setup.py script
-    ]
+    ],
 )
 def test_editable_with_pyproject(tmp_path, venv, files, editable_opts):
     project = tmp_path / "mypkg"
     project.mkdir()
     jaraco.path.build(files, prefix=project)
 
-    cmd = [venv.exe(), "-m", "pip", "install",
-           "--no-build-isolation",  # required to force current version of setuptools
-           "-e", str(project), *editable_opts]
+    cmd = [
+        venv.exe(),
+        "-m",
+        "pip",
+        "install",
+        "--no-build-isolation",  # required to force current version of setuptools
+        "-e",
+        str(project),
+        *editable_opts,
+    ]
     print(str(subprocess.check_output(cmd), "utf-8"))
 
     cmd = [venv.exe(), "-m", "mypkg"]
     assert subprocess.check_output(cmd).strip() == b"3.14159.post0 Hello World"
 
-    (project / "src/mypkg/data.txt").write_text("foobar")
-    (project / "src/mypkg/mod.py").write_text("x = 42")
+    (project / "src/mypkg/data.txt").write_text("foobar", encoding="utf-8")
+    (project / "src/mypkg/mod.py").write_text("x = 42", encoding="utf-8")
     assert subprocess.check_output(cmd).strip() == b"3.14159.post0 foobar 42"
 
 
 def test_editable_with_flat_layout(tmp_path, venv, editable_opts):
     files = {
         "mypkg": {
-            "pyproject.toml": dedent("""\
+            "pyproject.toml": dedent(
+                """\
                 [build-system]
                 requires = ["setuptools", "wheel"]
                 build-backend = "setuptools.build_meta"
@@ -151,7 +165,8 @@ def test_editable_with_flat_layout(tmp_path, venv, editable_opts):
                 [tool.setuptools]
                 packages = ["pkg"]
                 py-modules = ["mod"]
-                """),
+                """
+            ),
             "pkg": {"__init__.py": "a = 4"},
             "mod.py": "b = 2",
         },
@@ -159,9 +174,16 @@ def test_editable_with_flat_layout(tmp_path, venv, editable_opts):
     jaraco.path.build(files, prefix=tmp_path)
     project = tmp_path / "mypkg"
 
-    cmd = [venv.exe(), "-m", "pip", "install",
-           "--no-build-isolation",  # required to force current version of setuptools
-           "-e", str(project), *editable_opts]
+    cmd = [
+        venv.exe(),
+        "-m",
+        "pip",
+        "install",
+        "--no-build-isolation",  # required to force current version of setuptools
+        "-e",
+        str(project),
+        *editable_opts,
+    ]
     print(str(subprocess.check_output(cmd), "utf-8"))
     cmd = [venv.exe(), "-c", "import pkg, mod; print(pkg.a, mod.b)"]
     assert subprocess.check_output(cmd).strip() == b"4 2"
@@ -170,7 +192,8 @@ def test_editable_with_flat_layout(tmp_path, venv, editable_opts):
 def test_editable_with_single_module(tmp_path, venv, editable_opts):
     files = {
         "mypkg": {
-            "pyproject.toml": dedent("""\
+            "pyproject.toml": dedent(
+                """\
                 [build-system]
                 requires = ["setuptools", "wheel"]
                 build-backend = "setuptools.build_meta"
@@ -181,38 +204,83 @@ def test_editable_with_single_module(tmp_path, venv, editable_opts):
 
                 [tool.setuptools]
                 py-modules = ["mod"]
-                """),
+                """
+            ),
             "mod.py": "b = 2",
         },
     }
     jaraco.path.build(files, prefix=tmp_path)
     project = tmp_path / "mypkg"
 
-    cmd = [venv.exe(), "-m", "pip", "install",
-           "--no-build-isolation",  # required to force current version of setuptools
-           "-e", str(project), *editable_opts]
+    cmd = [
+        venv.exe(),
+        "-m",
+        "pip",
+        "install",
+        "--no-build-isolation",  # required to force current version of setuptools
+        "-e",
+        str(project),
+        *editable_opts,
+    ]
     print(str(subprocess.check_output(cmd), "utf-8"))
     cmd = [venv.exe(), "-c", "import mod; print(mod.b)"]
     assert subprocess.check_output(cmd).strip() == b"2"
 
 
 class TestLegacyNamespaces:
-    """Ported from test_develop"""
+    # legacy => pkg_resources.declare_namespace(...) + setup(namespace_packages=...)
 
-    def test_namespace_package_importable(self, venv, tmp_path, editable_opts):
+    def test_nspkg_file_is_unique(self, tmp_path, monkeypatch):
+        deprecation = pytest.warns(
+            SetuptoolsDeprecationWarning, match=".*namespace_packages parameter.*"
+        )
+        installation_dir = tmp_path / ".installation_dir"
+        installation_dir.mkdir()
+        examples = (
+            "myns.pkgA",
+            "myns.pkgB",
+            "myns.n.pkgA",
+            "myns.n.pkgB",
+        )
+
+        for name in examples:
+            pkg = namespaces.build_namespace_package(tmp_path, name, version="42")
+            with deprecation, monkeypatch.context() as ctx:
+                ctx.chdir(pkg)
+                dist = run_setup("setup.py", stop_after="config")
+                cmd = editable_wheel(dist)
+                cmd.finalize_options()
+                editable_name = cmd.get_finalized_command("dist_info").name
+                cmd._install_namespaces(installation_dir, editable_name)
+
+        files = list(installation_dir.glob("*-nspkg.pth"))
+        assert len(files) == len(examples)
+
+    @pytest.mark.parametrize(
+        "impl",
+        (
+            "pkg_resources",
+            #  "pkgutil",  => does not work
+        ),
+    )
+    @pytest.mark.parametrize("ns", ("myns.n",))
+    def test_namespace_package_importable(
+        self, venv, tmp_path, ns, impl, editable_opts
+    ):
         """
         Installing two packages sharing the same namespace, one installed
         naturally using pip or `--single-version-externally-managed`
         and the other installed in editable mode should leave the namespace
         intact and both packages reachable by import.
+        (Ported from test_develop).
         """
         build_system = """\
         [build-system]
         requires = ["setuptools"]
         build-backend = "setuptools.build_meta"
         """
-        pkg_A = namespaces.build_namespace_package(tmp_path, 'myns.pkgA')
-        pkg_B = namespaces.build_namespace_package(tmp_path, 'myns.pkgB')
+        pkg_A = namespaces.build_namespace_package(tmp_path, f"{ns}.pkgA", impl=impl)
+        pkg_B = namespaces.build_namespace_package(tmp_path, f"{ns}.pkgB", impl=impl)
         (pkg_A / "pyproject.toml").write_text(build_system, encoding="utf-8")
         (pkg_B / "pyproject.toml").write_text(build_system, encoding="utf-8")
         # use pip to install to the target directory
@@ -220,7 +288,7 @@ class TestLegacyNamespaces:
         opts.append("--no-build-isolation")  # force current version of setuptools
         venv.run(["python", "-m", "pip", "install", str(pkg_A), *opts])
         venv.run(["python", "-m", "pip", "install", "-e", str(pkg_B), *opts])
-        venv.run(["python", "-c", "import myns.pkgA; import myns.pkgB"])
+        venv.run(["python", "-c", f"import {ns}.pkgA; import {ns}.pkgB"])
         # additionally ensure that pkg_resources import works
         venv.run(["python", "-c", "import pkg_resources"])
 
@@ -245,7 +313,8 @@ class TestPep420Namespaces:
         """Currently users can create a namespace by tweaking `package_dir`"""
         files = {
             "pkgA": {
-                "pyproject.toml": dedent("""\
+                "pyproject.toml": dedent(
+                    """\
                     [build-system]
                     requires = ["setuptools", "wheel"]
                     build-backend = "setuptools.build_meta"
@@ -256,7 +325,8 @@ class TestPep420Namespaces:
 
                     [tool.setuptools]
                     package-dir = {"myns.n.pkgA" = "src"}
-                    """),
+                    """
+                ),
                 "src": {"__init__.py": "a = 1"},
             },
         }
@@ -281,7 +351,8 @@ class TestPep420Namespaces:
         """
         files = {
             "pkgA": {
-                "pyproject.toml": dedent("""\
+                "pyproject.toml": dedent(
+                    """\
                     [build-system]
                     requires = ["setuptools", "wheel"]
                     build-backend = "setuptools.build_meta"
@@ -292,7 +363,8 @@ class TestPep420Namespaces:
 
                     [tool.setuptools]
                     packages.find.include = ["mypkg.*"]
-                    """),
+                    """
+                ),
                 "mypkg": {
                     "__init__.py": "",
                     "other.py": "b = 1",
@@ -300,7 +372,7 @@ class TestPep420Namespaces:
                         "__init__.py": "",
                         "pkgA.py": "a = 1",
                     },
-                 },
+                },
                 "MANIFEST.in": EXAMPLE["MANIFEST.in"],
             },
         }
@@ -342,7 +414,7 @@ def test_editable_with_prefix(tmp_path, sample_project, editable_opts):
     site_packages.mkdir(parents=True)
 
     # install workaround
-    pip_run.launch.inject_sitecustomize(site_packages)
+    _addsitedir(site_packages)
 
     env = dict(os.environ, PYTHONPATH=str(site_packages))
     cmd = [
@@ -362,8 +434,6 @@ def test_editable_with_prefix(tmp_path, sample_project, editable_opts):
     # now run 'sample' with the prefix on the PYTHONPATH
     bin = 'Scripts' if platform.system() == 'Windows' else 'bin'
     exe = prefix / bin / 'sample'
-    if sys.version_info < (3, 8) and platform.system() == 'Windows':
-        exe = str(exe)
     subprocess.check_call([exe], env=env)
 
 
@@ -372,6 +442,7 @@ class TestFinderTemplate:
     If at some point in time the implementation is changed for something different,
     this test can be modified or even excluded.
     """
+
     def install_finder(self, finder):
         loc = {}
         exec(finder, loc, loc)
@@ -391,7 +462,7 @@ class TestFinderTemplate:
 
         mapping = {
             "pkg1": str(tmp_path / "src1/pkg1"),
-            "mod2": str(tmp_path / "src2/mod2")
+            "mod2": str(tmp_path / "src2/mod2"),
         }
         template = _finder_template(str(uuid4()), mapping, {})
 
@@ -522,7 +593,7 @@ class TestFinderTemplate:
                 "__init__.py": "",
                 "bar": {
                     "__init__.py": "",
-                }
+                },
             },
         }
         jaraco.path.build(files, prefix=tmp_path)
@@ -540,6 +611,132 @@ class TestFinderTemplate:
             with pytest.raises(ImportError, match="foobar"):
                 import_module("foobar")
 
+    def test_case_sensitivity(self, tmp_path):
+        files = {
+            "foo": {
+                "__init__.py": "",
+                "lowercase.py": "x = 1",
+                "bar": {
+                    "__init__.py": "",
+                    "lowercase.py": "x = 2",
+                },
+            },
+        }
+        jaraco.path.build(files, prefix=tmp_path)
+        mapping = {
+            "foo": str(tmp_path / "foo"),
+        }
+        template = _finder_template(str(uuid4()), mapping, {})
+        with contexts.save_paths(), contexts.save_sys_modules():
+            sys.modules.pop("foo", None)
+
+            self.install_finder(template)
+            with pytest.raises(ImportError, match="'FOO'"):
+                import_module("FOO")
+
+            with pytest.raises(ImportError, match="'foo\\.LOWERCASE'"):
+                import_module("foo.LOWERCASE")
+
+            with pytest.raises(ImportError, match="'foo\\.bar\\.Lowercase'"):
+                import_module("foo.bar.Lowercase")
+
+            with pytest.raises(ImportError, match="'foo\\.BAR'"):
+                import_module("foo.BAR.lowercase")
+
+            with pytest.raises(ImportError, match="'FOO'"):
+                import_module("FOO.bar.lowercase")
+
+            mod = import_module("foo.lowercase")
+            assert mod.x == 1
+
+            mod = import_module("foo.bar.lowercase")
+            assert mod.x == 2
+
+    def test_namespace_case_sensitivity(self, tmp_path):
+        files = {
+            "pkg": {
+                "__init__.py": "a = 13",
+                "foo": {
+                    "__init__.py": "b = 37",
+                    "bar.py": "c = 42",
+                },
+            },
+        }
+        jaraco.path.build(files, prefix=tmp_path)
+
+        mapping = {"ns.othername": str(tmp_path / "pkg")}
+        namespaces = {"ns": []}
+
+        template = _finder_template(str(uuid4()), mapping, namespaces)
+        with contexts.save_paths(), contexts.save_sys_modules():
+            for mod in ("ns", "ns.othername"):
+                sys.modules.pop(mod, None)
+
+            self.install_finder(template)
+            pkg = import_module("ns.othername")
+            expected = str((tmp_path / "pkg").resolve())
+            assert_path(pkg, expected)
+            assert pkg.a == 13
+
+            foo = import_module("ns.othername.foo")
+            assert foo.b == 37
+
+            bar = import_module("ns.othername.foo.bar")
+            assert bar.c == 42
+
+            with pytest.raises(ImportError, match="'NS'"):
+                import_module("NS.othername.foo")
+
+            with pytest.raises(ImportError, match="'ns\\.othername\\.FOO\\'"):
+                import_module("ns.othername.FOO")
+
+            with pytest.raises(ImportError, match="'ns\\.othername\\.foo\\.BAR\\'"):
+                import_module("ns.othername.foo.BAR")
+
+    def test_intermediate_packages(self, tmp_path):
+        """
+        The finder should not import ``fullname`` if the intermediate segments
+        don't exist (see pypa/setuptools#4019).
+        """
+        files = {
+            "src": {
+                "mypkg": {
+                    "__init__.py": "",
+                    "config.py": "a = 13",
+                    "helloworld.py": "b = 13",
+                    "components": {
+                        "config.py": "a = 37",
+                    },
+                },
+            }
+        }
+        jaraco.path.build(files, prefix=tmp_path)
+
+        mapping = {"mypkg": str(tmp_path / "src/mypkg")}
+        template = _finder_template(str(uuid4()), mapping, {})
+
+        with contexts.save_paths(), contexts.save_sys_modules():
+            for mod in (
+                "mypkg",
+                "mypkg.config",
+                "mypkg.helloworld",
+                "mypkg.components",
+                "mypkg.components.config",
+                "mypkg.components.helloworld",
+            ):
+                sys.modules.pop(mod, None)
+
+            self.install_finder(template)
+
+            config = import_module("mypkg.components.config")
+            assert config.a == 37
+
+            helloworld = import_module("mypkg.helloworld")
+            assert helloworld.b == 13
+
+            with pytest.raises(ImportError):
+                import_module("mypkg.components.helloworld")
+
 
 def test_pkg_roots(tmp_path):
     """This test focus in getting a particular implementation detail right.
@@ -558,7 +755,7 @@ def test_pkg_roots(tmp_path):
     package_dir = {
         "a.b.c": "other",
         "a.b.c.x.y.z": "another",
-        "m.n.o.p.q": "yet_another"
+        "m.n.o.p.q": "yet_another",
     }
     packages = [
         "a",
@@ -625,13 +822,16 @@ class TestOverallBehaviour:
             "src": {"mypkg": FLAT_LAYOUT["mypkg"]},
         },
         "custom-layout": {
-            "pyproject.toml": dedent(PYPROJECT) + dedent("""\
+            "pyproject.toml": dedent(PYPROJECT)
+            + dedent(
+                """\
                 [tool.setuptools]
                 packages = ["mypkg", "mypkg.subpackage"]
 
                 [tool.setuptools.package-dir]
                 "mypkg.subpackage" = "other"
-                """),
+                """
+            ),
             "MANIFEST.in": EXAMPLE["MANIFEST.in"],
             "otherfile.py": "",
             "mypkg": {
@@ -707,7 +907,8 @@ class TestOverallBehaviour:
 
 class TestLinkTree:
     FILES = deepcopy(TestOverallBehaviour.EXAMPLES["src-layout"])
-    FILES["pyproject.toml"] += dedent("""\
+    FILES["pyproject.toml"] += dedent(
+        """\
         [tool.setuptools]
         # Temporary workaround: both `include-package-data` and `package-data` configs
         # can be removed after #3260 is fixed.
@@ -717,7 +918,8 @@ class TestLinkTree:
         [tool.setuptools.packages.find]
         where = ["src"]
         exclude = ["*.subpackage*"]
-        """)
+        """
+    )
     FILES["src"]["mypkg"]["resource.not_in_manifest"] = "abc"
 
     def test_generated_tree(self, tmp_path):
@@ -822,25 +1024,31 @@ def test_compat_install(tmp_path, venv):
 def test_pbr_integration(tmp_path, venv, editable_opts):
     """Ensure editable installs work with pbr, issue #3500"""
     files = {
-        "pyproject.toml": dedent("""\
+        "pyproject.toml": dedent(
+            """\
             [build-system]
             requires = ["setuptools"]
             build-backend = "setuptools.build_meta"
-            """),
-        "setup.py": dedent("""\
+            """
+        ),
+        "setup.py": dedent(
+            """\
             __import__('setuptools').setup(
                 pbr=True,
                 setup_requires=["pbr"],
             )
-            """),
-        "setup.cfg": dedent("""\
+            """
+        ),
+        "setup.cfg": dedent(
+            """\
             [metadata]
             name = mypkg
 
             [files]
             packages =
                 mypkg
-            """),
+            """
+        ),
         "mypkg": {
             "__init__.py": "",
             "hello.py": "print('Hello world!')",
@@ -866,11 +1074,13 @@ class TestCustomBuildPy:
     During the transition period setuptools should prevent potential errors from
     happening due to those assumptions.
     """
+
     # TODO: Remove tests after _run_build_steps is removed.
 
     FILES = {
         **TestOverallBehaviour.EXAMPLES["flat-layout"],
-        "setup.py": dedent("""\
+        "setup.py": dedent(
+            """\
             import pathlib
             from setuptools import setup
             from setuptools.command.build_py import build_py as orig
@@ -881,7 +1091,8 @@ class TestCustomBuildPy:
                     raise ValueError("TEST_RAISE")
 
             setup(cmdclass={"build_py": my_build_py})
-            """),
+            """
+        ),
     }
 
     def test_safeguarded_from_errors(self, tmp_path, venv):
@@ -969,11 +1180,18 @@ def test_debugging_tips(tmpdir_cwd, monkeypatch):
     simulated_failure = Mock(side_effect=SimulatedErr())
     monkeypatch.setattr(cmd, "get_finalized_command", simulated_failure)
 
-    expected_msg = "following steps are recommended to help debugging"
+    expected_msg = "following steps are recommended to help debug"
     with pytest.raises(SimulatedErr), pytest.warns(_DebuggingTips, match=expected_msg):
         cmd.run()
 
 
+@pytest.mark.filterwarnings("error")
+def test_encode_pth():
+    """Ensure _encode_pth function does not produce encoding warnings"""
+    content = _encode_pth("tkmilan_ç_utf8")  # no warnings (would be turned into errors)
+    assert isinstance(content, bytes)
+
+
 def install_project(name, venv, tmp_path, files, *opts):
     project = tmp_path / name
     project.mkdir()
@@ -986,6 +1204,16 @@ def install_project(name, venv, tmp_path, files, *opts):
     return project, out
 
 
+def _addsitedir(new_dir: Path):
+    """To use this function, it is necessary to insert new_dir in front of sys.path.
+    The Python process will try to import a ``sitecustomize`` module on startup.
+    If we manipulate sys.path/PYTHONPATH, we can force it to run our code,
+    which invokes ``addsitedir`` and ensure ``.pth`` files are loaded.
+    """
+    file = f"import site; site.addsitedir({os.fspath(new_dir)!r})\n"
+    (new_dir / "sitecustomize.py").write_text(file, encoding="utf-8")
+
+
 # ---- Assertion Helpers ----
 
 
